@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -38,10 +38,9 @@
 
 #include "version.h"
 
-struct rule_struct *rulestruct;
-struct ref_struct *refstruct;
+struct _Rule_Struct *rulestruct;
 
-void sagan_alert( SaganEvent *Event ) { 
+void sagan_alert( _SaganEvent *Event ) { 
 
 char tmpref[2048]="";
 
