@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -44,13 +44,9 @@
 #include "sagan-esmtp.h"
 #include "version.h"
 
-struct _SaganCounters *counters;
+struct _Rule_Struct *rulestruct;
 
-struct rule_struct *rulestruct;
-
-void sagan_esmtp_thread (SaganEvent *Event) { 
-
-pthread_mutex_t email_mutex = PTHREAD_MUTEX_INITIALIZER;
+void sagan_esmtp_thread (_SaganEvent *Event) { 
 
 char tmpref[2048];
 char tmpemail[255];
@@ -154,7 +150,6 @@ if (!smtp_start_session (session)) {
     */
 
 	sagan_log(Event->config, 0, "[%s, line %d] SMTP Error: %s", __FILE__, __LINE__, smtp_strerror (smtp_errno (), errtmp, sizeof(errtmp)));
-	counters->saganesmtpdrop++;
 
    } else {
 
@@ -170,12 +165,6 @@ failure:
 if(session != NULL)
 	smtp_destroy_session (session);
 
-pthread_mutex_lock ( &email_mutex );
-counters->threademailc--;
-pthread_mutex_unlock ( &email_mutex );
-
-pthread_exit(NULL);
-
 }
 
 int
