@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Softwink, Inc. 
-** Copyright (C) 2009-2011 Champ Clark III <champ@softwink.com>
+** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -26,7 +26,7 @@
 
 #ifdef HAVE_LIBESMTP
 
-int fixlf(char *, char *);
+int fixlf(_SaganConfig *, char *, char *);
 
 #endif
 
