@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -41,10 +41,9 @@
 
 #include "version.h"
 
-struct _SaganCounters *counters;
-struct rule_struct *rulestruct;
+struct _Rule_Struct *rulestruct;
 
-void sagan_ext_thread ( SaganEvent *Event ) {
+void sagan_ext_thread ( _SaganEvent *Event ) {
 
 pthread_mutex_t ext_mutex = PTHREAD_MUTEX_INITIALIZER;
 
@@ -132,10 +131,5 @@ if (( pid = fork()) == 0 ) {
 
    waitpid(pid, NULL, 0);
 
-pthread_mutex_lock( &ext_mutex );
-counters->threadextc--;
-pthread_mutex_unlock( &ext_mutex );
-
-pthread_exit(NULL);
 }
 
