@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -46,8 +46,6 @@
 
 #include "sagan-prelude.h"
 
-struct _SaganCounters *counters;
-
 #define ANALYZER_CLASS "Log Analyzer"
 #define ANALYZER_MODEL "Sagan"
 #define ANALYZER_MANUFACTURER "http://sagan.quadrantsec.com"
@@ -57,7 +55,7 @@ struct _SaganCounters *counters;
 
 prelude_client_t *preludeclient;
 
-struct rule_struct *rulestruct;
+struct _Rule_Struct *rulestruct;
 
 /* Init the Prelude sub system. */
 
@@ -173,11 +171,9 @@ int add_int_data( _SaganConfig *config, idmef_alert_t *alert, const char *meanin
 /* sagan_prelude() - This is the sub/thread called from the main process    */
 /****************************************************************************/
 
-void sagan_prelude( SaganEvent *Event ) 
+void sagan_prelude( _SaganEvent *Event ) 
 {
 
-pthread_mutex_t prelude_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 int ret;
 
 int sid=0;
@@ -247,11 +243,6 @@ prelude_client_send_idmef(preludeclient, idmef);
 err:
 idmef_message_destroy(idmef);
 
-pthread_mutex_lock ( &prelude_mutex );
-counters->threadpreludec--;
-pthread_mutex_unlock ( &prelude_mutex );
-
-pthread_exit(NULL);
 }
 
 /* Assigns severity to an event.  For example,  priority 1 == High */
