@@ -26,10 +26,8 @@
 
 #ifdef HAVE_LIBPRELUDE
 
-void PreludeInit( _SaganConfig *);
 int setup_analyzer(idmef_analyzer_t *);
 int add_int_data(_SaganConfig *, idmef_alert_t *, const char *, uint32_t );
-void sagan_prelude( SaganEvent * );
 int event_to_impact(_SaganConfig *, int, idmef_alert_t *);
 int event_to_reference(char *, idmef_classification_t *);
 int event_to_source_target(_SaganConfig *, char *, char *, int ,int ,int , idmef_alert_t *);
