@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -18,7 +18,7 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* sagan-db.c 
+/* sagan-snort.c 
  *
  * Threaded function for database support.   These functions are for both
  * MySQL and PostgreSQL.   These allow Sagan to report to Snort databases
@@ -64,12 +64,10 @@ char pgconnect[2048];
 #endif
 
 struct _SaganCounters *counters;
-
-struct rule_struct *rulestruct;
+struct _Rule_Struct *rulestruct;
 
 pthread_mutex_t db_mutex;
 
-
 /********************************************/
 /* Connection to various types of databases */
 /********************************************/
@@ -109,7 +107,7 @@ mysql_options(mysql,MYSQL_READ_DEFAULT_GROUP,config->dbname);
 
 mysql_options(mysql,MYSQL_OPT_RECONNECT, &reconnect);
 
-if (!mysql_real_connect(mysql, dbh, dbu, dbp, dbn, MYSQL_PORT, NULL, 0)) {
+if (!mysql_real_connect(mysql, dbh, dbu, dbp, dbn, config->dbport, NULL, 0)) {
      sagan_log(config, 1, "[%s, line %d] MySQL Error %u: \"%s\"", __FILE__,  __LINE__, mysql_errno(mysql), mysql_error(mysql));
      }
 
@@ -159,11 +157,8 @@ char *re=NULL;		/* "return" point for row */
 int mysql_last_errno = 0; 
 int mysql_reconnect_count = 0;
 
-pthread_mutex_lock( &db_mutex );
-
 strlcpy(sqltmp, sql, sizeof(sqltmp));
 
-
 if ( debug->debugsql ) sagan_log(config, 0, "%s", sqltmp); 
 
 #ifdef HAVE_LIBMYSQLCLIENT_R
@@ -185,16 +180,17 @@ while ( mysql_real_query(mysql, sqltmp,  strlen(sqltmp)) != 0 ) {
 
 	 } else { 
 
-	removelockfile(config);
-        sagan_log(config, 1, "[%s, line %d] MySQL Error [%u:] \"%s\"\nOffending SQL statement: %s\n", __FILE__,  __LINE__, mysql_errno(mysql), mysql_error(mysql), sqltmp);
+        sagan_log(config, 0, "[%s, line %d] MySQL Error [%u] \"%s\"\n[*] Offending SQL statement: %s\n", __FILE__,  __LINE__, mysql_errno(mysql), mysql_error(mysql), sqltmp);
+	
+	return(0);
 	}
    
    }
 
 
-if ( mysql_reconnect_count != 0 ) { 			/* If there's a reconnect_count,  we must of lost connection */
+if ( mysql_reconnect_count != 0 ) { 				/* If there's a reconnect_count,  we must of lost connection */
    sagan_log(config, 0, "MySQL connection re-established!"); 	/* Log it */
-   mysql_reconnect_count=0;				/* Reset the counter */
+   mysql_reconnect_count=0;					/* Reset the counter */
    }
 
 res = mysql_use_result(mysql);
@@ -207,7 +203,6 @@ if ( res != NULL ) {
  }
 
 mysql_free_result(res);
-pthread_mutex_unlock( &db_mutex );
 return(re);
 }
 #else
@@ -238,7 +233,6 @@ if ( PQntuples(result) != 0 ) {
     }
 
 PQclear(result);
-pthread_mutex_unlock( &db_mutex);
 return(re);
 
 }
@@ -316,7 +310,7 @@ return(t_cid);
 /* Get signature ID.  If on doesn't exsist,  put one in. */
 /*********************************************************/
 
-int get_sig_sid ( SaganEvent *Event ) {
+int get_sig_sid ( _SaganEvent *Event ) {
 
 
 char sqltmp[MAXSQL];
@@ -348,7 +342,7 @@ sig_class_id = atoi(sqlout);
 
 /* Look for the signature id */
 
-snprintf(sqltmp, sizeof(sqltmp), "SELECT sig_id FROM signature WHERE sig_name='%s' AND sig_rev=%s AND sig_sid=%s", rulestruct[Event->found].s_msg, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid);
+snprintf(sqltmp, sizeof(sqltmp), "SELECT sig_id FROM signature WHERE sig_name='%s' AND sig_rev=%s AND sig_sid=%s", Event->f_msg, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid);
 sql=sqltmp;
 
 
@@ -358,12 +352,16 @@ sqlout = db_query( Event->debug, Event->config, sql );
 
 if ( sqlout == NULL ) {
 
-   snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO signature(sig_name, sig_class_id, sig_priority, sig_rev, sig_sid) VALUES ('%s', '%d', '%d', '%s', '%s' )", rulestruct[Event->found].s_msg, sig_class_id, rulestruct[Event->found].s_pri, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid );
+//   snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO signature(sig_name, sig_class_id, sig_priority, sig_rev, sig_sid) VALUES ('%s', '%d', '%d', '%s', '%s' )", rulestruct[Event->found].s_msg, sig_class_id, rulestruct[Event->found].s_pri, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid );
+
+// Event->f_msg
+
+ snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO signature(sig_name, sig_class_id, sig_priority, sig_rev, sig_sid) VALUES ('%s', '%d', '%d', '%s', '%s' )", Event->f_msg, sig_class_id, rulestruct[Event->found].s_pri, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid );
    sql=sqltmp;
    db_query( Event->debug, Event->config, sql );
 
    /* Get the new ID of the new entry */
-   snprintf(sqltmp, sizeof(sqltmp), "SELECT sig_id FROM signature WHERE sig_name='%s' AND sig_rev=%s AND sig_sid=%s", rulestruct[Event->found].s_msg, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid );
+   snprintf(sqltmp, sizeof(sqltmp), "SELECT sig_id FROM signature WHERE sig_name='%s' AND sig_rev=%s AND sig_sid=%s", Event->f_msg, rulestruct[Event->found].s_rev, rulestruct[Event->found].s_sid );
    sql=sqltmp;
    sqlout = db_query( Event->debug, Event->config, sql );
    }
@@ -378,19 +376,14 @@ return(t_sig_id);
 /* Insert into event table */
 /***************************/
 
-void insert_event ( SaganEvent *Event, int sig_sid, char *date,  char *time ) { 
+void insert_event ( _SaganEvent *Event, int sig_sid, char *date,  char *time ) { 
 
 char sqltmp[MAXSQL];
 char *sql;
 
-pthread_mutex_lock( &db_mutex );
-
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO event(sid, cid, signature, timestamp) VALUES ('%d', '%" PRIu64 "', '%d', '%s %s')", Event->config->sensor_id, Event->cid, sig_sid, date, time );
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO event(sid, cid, signature, timestamp) VALUES ('%d', '%" PRIu64 "', '%d', '%s %s')", Event->config->sensor_id, counters->cid, sig_sid, date, time );
 sql=sqltmp;
-
-pthread_mutex_unlock( &db_mutex );
-
-db_query( Event->debug, Event->config, sql );
+db_query( Event->debug, Event->config, sql ); 
 
 }
 
@@ -399,7 +392,7 @@ db_query( Event->debug, Event->config, sql );
 /* Insert data into iphdr and tcphdr - most of this is bogus as we're not really TCP/IP */
 /****************************************************************************************/
 
-void insert_hdr ( SaganEvent *Event,  char *ipsrc, char *ipdst )  {
+void insert_hdr ( _SaganEvent *Event,  char *ipsrc, char *ipdst )  {
 
 char sqltmp[MAXSQL];
 char *sql;
@@ -410,14 +403,14 @@ int ipproto = rulestruct[Event->found].ip_proto;
 
 /* 4 == IPv4 */
 
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO iphdr VALUES ( '%d', '%" PRIu64 "', '%u', '%u', '4', '0', '0', '0', '0', '0', '0', '0', '%d', '0' )", Event->config->sensor_id, Event->cid, ip2bit(Event->config, ipsrc ), ip2bit(Event->config, ipdst), ipproto );
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO iphdr VALUES ( '%d', '%" PRIu64 "', '%u', '%u', '4', '0', '0', '0', '0', '0', '0', '0', '%d', '0' )", Event->config->sensor_id, counters->cid, ip2bit(Event->config, ipsrc ), ip2bit(Event->config, ipdst), ipproto );
 
 sql=sqltmp;
 db_query( Event->debug, Event->config, sql );
 
 /* "tcp" */
 if ( ipproto == 6 )  {
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO tcphdr VALUES ( '%d', '%" PRIu64 "', '%d', '%d', '0', '0', '0', '0', '0', '0', '0', '0'  )", Event->config->sensor_id, Event->cid, Event->src_port, Event->dst_port  );
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO tcphdr VALUES ( '%d', '%" PRIu64 "', '%d', '%d', '0', '0', '0', '0', '0', '0', '0', '0'  )", Event->config->sensor_id, counters->cid, Event->src_port, Event->dst_port  );
 sql=sqltmp;
 db_query( Event->debug, Event->config, sql );
 } 
@@ -425,7 +418,7 @@ db_query( Event->debug, Event->config, sql );
 /* "udp" */
 
 if ( ipproto == 17 )  {
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO udphdr VALUES ( '%d', '%" PRIu64 "', '%d', '%d', '0', '0' )", Event->config->sensor_id, Event->cid, Event->src_port, Event->dst_port  );
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO udphdr VALUES ( '%d', '%" PRIu64 "', '%d', '%d', '0', '0' )", Event->config->sensor_id, counters->cid, Event->src_port, Event->dst_port  );
 sql=sqltmp;
 db_query( Event->debug, Event->config, sql );
 }
@@ -434,7 +427,7 @@ db_query( Event->debug, Event->config, sql );
 /* May expand on this if there's actually a use for it */
 
 if ( ipproto == 1 ) { 
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO icmphdr VALUES ( '%d', '%" PRIu64 "', '8', '8', '0', '0', '0' )", Event->config->sensor_id, Event->cid );
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO icmphdr VALUES ( '%d', '%" PRIu64 "', '8', '8', '0', '0', '0' )", Event->config->sensor_id, counters->cid );
 sql=sqltmp;
 db_query( Event->debug, Event->config, sql );
 }
@@ -446,15 +439,13 @@ db_query( Event->debug, Event->config, sql );
 /* Insert into payload table */
 /*****************************/
 
-void insert_payload ( SaganEvent *Event,  char *t_hex_data ) { 
+void insert_payload ( _SaganEvent *Event,  char *t_hex_data ) { 
 
 char sqltmp[MAXSQL]; 
 char *sql;
 
-pthread_mutex_lock( &db_mutex );
-snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO data(sid, cid, data_payload) VALUES ('%d', '%" PRIu64 "', '%s')", Event->config->sensor_id, Event->cid, t_hex_data);
+snprintf(sqltmp, sizeof(sqltmp), "INSERT INTO data(sid, cid, data_payload) VALUES ('%d', '%" PRIu64 "', '%s')", Event->config->sensor_id, counters->cid, t_hex_data);
 sql=sqltmp;
-pthread_mutex_unlock( &db_mutex );
 db_query( Event->debug, Event->config, sql );
 
 }
@@ -463,12 +454,12 @@ db_query( Event->debug, Event->config, sql );
 /* Record last cid */
 /*******************/
 
-void record_last_cid ( _SaganDebug *debug, _SaganConfig *config )  { 
+void record_last_cid ( _SaganDebug *debug, _SaganConfig *config, _SaganCounters *counters )  { 
 
 char sqltmp[MAXSQL];
 char *sql;
 
-snprintf(sqltmp, sizeof(sqltmp), "UPDATE sensor SET last_cid='%" PRIu64 "' where sid=%d and hostname='%s' and interface='%s' and filter='%s' and detail=%d", counters->sigcid, config->sensor_id, config->sagan_hostname, config->sagan_interface, config->sagan_filter, config->sagan_detail);
+snprintf(sqltmp, sizeof(sqltmp), "UPDATE sensor SET last_cid='%" PRIu64 "' where sid=%d and hostname='%s' and interface='%s' and filter='%s' and detail=%d", counters->cid - 1, config->sensor_id, config->sagan_hostname, config->sagan_interface, config->sagan_filter, config->sagan_detail);
 sql=sqltmp;
 db_query( debug, config, sql );
 
@@ -560,7 +551,7 @@ if ( sqlout == NULL )  {
 /* Snort specific thread code                                              */
 /***************************************************************************/
 
-void sagan_db_thread( SaganEvent *Event ) {
+void sagan_db_thread( _SaganEvent *Event ) {
 
 int sig_sid;
 int i;
@@ -573,6 +564,14 @@ char ip_dsttmp[65];
 char time[30];
 char date[30];
 
+/* We do the mutex_lock(db_mutex) here to avoid a race condition.  It was
+ * possible for two threads to have the same cid,  leading to dup entries
+ * in the SQL database  - Champ Clark III 04/03/2012
+ */
+
+pthread_mutex_lock( &db_mutex );
+counters->cid++;
+
 snprintf(message, sizeof(message), "%s", Event->message); 
 snprintf(ip_srctmp, sizeof(ip_srctmp), "%s", Event->ip_src);
 snprintf(ip_dsttmp, sizeof(ip_dsttmp), "%s", Event->ip_dst);
@@ -591,11 +590,8 @@ for (i = 0; i < rulestruct[Event->found].ref_count; i++ ) {
    query_reference( Event->debug, Event->config, rulestruct[Event->found].s_reference[i], rulestruct[Event->found].s_sid, sig_sid, i );
    }
 
-pthread_mutex_lock( &db_mutex );
-counters->threaddbc--;
 pthread_mutex_unlock( &db_mutex );
 
-pthread_exit(NULL);
 }
 
 #endif
