@@ -27,30 +27,16 @@
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
 
-int  db_connect( _SaganConfig * );
+void record_last_cid ( _SaganDebug *, _SaganConfig *, _SaganCounters * );
 
-char *db_query ( _SaganDebug *, _SaganConfig *,  char * );
+int  get_sig_sid( _SaganEvent *); 
 
-//int  get_sensor_id ( _SaganDebug *, char *,  char *,  char *,  int , int  );
+void insert_event (_SaganEvent *, int, char *,  char * );
 
-int  get_sensor_id ( _SaganDebug *, _SaganConfig *);
-
-uint64_t get_cid ( _SaganDebug *,  _SaganConfig * );
-
-void record_last_cid ( _SaganDebug *, _SaganConfig * );
-
-//int  get_sig_sid( _SaganDebug *, char *,  char *,
-//                  char *,  char *,
-//                  int         ,  int  );
-
-int  get_sig_sid( SaganEvent *); 
-
-void insert_event ( SaganEvent *, int, char *,  char * );
-
-void insert_hdr ( SaganEvent *,  char *, char * );
+void insert_hdr (_SaganEvent *,  char *, char * );
 
 
-void insert_payload( SaganEvent *, char *); 
+void insert_payload(_SaganEvent *, char *); 
 
 void query_reference ( _SaganDebug *, _SaganConfig *, char *, char *, int, int );
 
