@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -49,8 +49,7 @@
 #include "sagan.h"
 
 struct _SaganCounters *counters;
-
-struct class_struct *classstruct;
+struct _Class_Struct *classstruct;
 
 void load_classifications( _SaganDebug *debug, _SaganConfig *config, const char *ruleset )  { 
 
@@ -81,7 +80,7 @@ while(fgets(classbuf, sizeof(classbuf), classfile) != NULL) {
      continue;
      } else { 
      /* Allocate memory for classifications,  but not comments */
-     classstruct = (class_struct *) realloc(classstruct, (counters->classcount+1) * sizeof(class_struct));
+     classstruct = (_Class_Struct *) realloc(classstruct, (counters->classcount+1) * sizeof(_Class_Struct));
      }
 
      firststring = strtok_r(classbuf, ":", &saveptr);
