@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -65,7 +65,7 @@ struct liblognorm_struct *liblognormstruct;
 int liblognorm_count;
 #endif
 
-struct rule_struct *rulestruct;
+struct _Rule_Struct *rulestruct;
 struct _SaganCounters *counters;
 
 void load_config( _SaganDebug *debug, _SaganConfig *config  ) { 
@@ -93,26 +93,13 @@ int i,check;
 snprintf(config->sagan_alert_filepath, sizeof(config->sagan_alert_filepath), "%s", ALERTLOG);
 snprintf(config->sagan_lockfile, sizeof(config->sagan_lockfile), "%s", LOCKFILE);
 snprintf(config->sagan_log_path, sizeof(config->sagan_log_path), "%s", SAGANLOGPATH);
-
-config->max_external_threads=MAX_EXT_THREADS;
-
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-config->maxdb_threads=MAX_DB_THREADS;
-#endif
-
-#ifdef HAVE_LIBESMTP
-config->max_email_threads=MAX_EMAIL_THREADS; 
-#endif
-
-#ifdef HAVE_LIBPRELUDE
-config->max_prelude_threads=MAX_PRELUDE_THREADS;
-#endif
-
 config->sagan_proto = 17;		/* Default to UDP */
+config->max_output_threads = MAX_OUTPUT_THREADS;
+config->max_processor_threads = MAX_PROCESSOR_THREADS;
 
 /* Start loading configuration */
 
-rulestruct = (rule_struct *) realloc(rulestruct, (counters->rulecount+1) * sizeof(rule_struct));
+rulestruct = (_Rule_Struct *) realloc(rulestruct, (counters->rulecount+1) * sizeof(_Rule_Struct));
 
 /* Gather information for the master configuration file */
 
@@ -130,15 +117,26 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
 
      sagan_option = strtok_r(tmpbuf, " ", &tok);
 
+     if (!strcmp(remrt(sagan_option), "max_output_threads")) { 
+	 sagan_var = strtok_r(NULL, " ", &tok);
+         config->max_output_threads = atol(sagan_var);
+	 }
+
+     if (!strcmp(remrt(sagan_option), "max_processor_threads")) {
+	 sagan_var = strtok_r(NULL, " ", &tok);
+         config->max_processor_threads = atol(sagan_var);
+         }
+
      if (!strcmp(remrt(sagan_option), "disable_dns_warnings")) { 
          sagan_log(config, 0, "Supressing DNS warnings");
          config->disable_dns_warnings = 1;
 	 }
 
-     if (!strcmp(sagan_option, "max_ext_threads")) {
-         sagan_var = strtok_r(NULL, " ", &tok);
-         config->max_external_threads = atoi(sagan_var);
-         }
+     if (!strcmp(remrt(sagan_option), "syslog_src_lookup")) { 
+         sagan_log(config, 0, "DNS lookup of source address supplied by syslog daemon");
+	 config->syslog_src_lookup = 1; 
+	 }
+
 
      if (!strcmp(sagan_option, "sagan_host")) {
         snprintf(config->sagan_host, sizeof(config->sagan_host)-1, "%s", strtok_r(NULL, " " , &tok));
@@ -150,6 +148,11 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
 	 config->sagan_port = atoi(sagan_var);
          }
 
+#ifndef HAVE_LIBESMTP
+if (!strcmp(sagan_option, "send-to") || !strcmp(sagan_option, "min_email_priority")) 
+   sagan_log(config,1, "\"libesmtp\" support not found. Re-compile with ESMTP support or disable in the sagan.conf.");
+#endif
+
 #ifdef HAVE_LIBESMTP
 
    if (!strcmp(sagan_option, "send-to")) { 
@@ -160,11 +163,6 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
       config->sagan_sendto_flag=1;
       }
 
-   if (!strcmp(sagan_option, "max_email_threads")) {
-       sagan_var = strtok_r(NULL, " ", &tok);
-       config->max_email_threads = atoi(sagan_var);
-       }
-
    if (!strcmp(sagan_option, "min_email_priority")) {
        sagan_var = strtok_r(NULL, " ", &tok);
        config->min_email_priority = atoi(sagan_var);
@@ -172,13 +170,9 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
 
 #endif
 
-#ifdef HAVE_LIBPRELUDE
-
-     if (!strcmp(sagan_option, "max_prelude_threads")) { 
-        sagan_var = strtok_r(NULL, " ", &tok); 
-	config->max_prelude_threads = atol(sagan_var);
-	}
-
+#ifndef HAVE_LIBPCAP
+if (!strcmp(sagan_option, "plog_interface") || !strcmp(sagan_option, "plog_logdev") || !strcmp(sagan_option, "plog_port")) 
+   sagan_log(config,1, "\"libpcap\" support not found. Re-compile with PCAP support or disable in the sagan.conf.");
 #endif
 
 #ifdef HAVE_LIBPCAP
@@ -205,11 +199,6 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
 
-     if (!strcmp(sagan_option, "maxdb_threads")) {
-        sagan_var = strtok_r(NULL, " " , &tok);
-        config->maxdb_threads = atol(sagan_var);
-        }
-
      if (!strcmp(sagan_option, "sagan_proto")) { 
         sagan_var = strtok_r(NULL, " ", &tok);
 	config->sagan_proto = atoi(sagan_var);
@@ -237,6 +226,13 @@ while(fgets(tmpbuf, sizeof(tmpbuf), sagancfg) != NULL) {
 
 #endif
 
+#ifndef HAVE_LIBLOGNORM
+if (!strcmp(sagan_option, "normalize:")) {
+   sagan_log(config, 0, "WARNING: Sagan was not compiled with \"liblognorm\" support!");
+   sagan_log(config, 0, "WARNING: Sagan will continue,  but _without_ liblognorm!");
+   }
+#endif
+
 #ifdef HAVE_LIBLOGNORM
 
 /*
@@ -272,6 +268,21 @@ if (!strcmp(sagan_option, "output")) {
 	config->sagan_ext_flag=1;
         }
 
+
+#ifdef WITH_SNORTSAM
+if (!strcmp(sagan_var, "alert_fwsam:")) { 
+       snprintf(config->sagan_fwsam_info, sizeof(config->sagan_fwsam_info), "%s", remrt(strtok_r(NULL, " ", &tok)));
+       config->sagan_fwsam_flag=1; 
+       }
+#endif
+
+#ifndef HAVE_LIBDNET
+if (!strcmp(sagan_var, "unified2:")) { 
+   sagan_log(config, 0,"\"libdnet\" support not found.  This is needed for unified2."); 
+   sagan_log(config, 1, "Re-compile with libdnet support or disable in the sagan.conf.");
+   }
+#endif
+
 #ifdef HAVE_LIBDNET
 
 if (!strcmp(sagan_var, "unified2:")) { 
@@ -302,9 +313,15 @@ if (!strcmp(sagan_var, "unified2:")) {
 
 #endif
 
+#ifndef HAVE_LIBPRELUDE
+	if (!strcmp(sagan_var, "prelude:"))
+           sagan_log(config,1, " libprelude support not found. Re-compile with Prelude support or disable in the sagan.conf.");
+#endif 
+
 #ifdef HAVE_LIBPRELUDE
 	
 	if (!strcmp(sagan_var, "prelude:")) { 
+	   config->output_thread_flag = 1; 
 	   ptmp = sagan_var; 
 
 	   while (ptmp != NULL ) { 
@@ -325,6 +342,7 @@ if (!strcmp(sagan_var, "unified2:")) {
 #ifdef HAVE_LIBESMTP
 
 	if (!strcmp(sagan_var, "email:")) { 
+	   config->output_thread_flag = 1;
 	   ptmp = sagan_var;
 
 	   while (ptmp != NULL ) { 
@@ -353,6 +371,7 @@ if (!strcmp(sagan_var, "unified2:")) {
 	/* output type (database, etc) */
 
 	if (!strcmp(sagan_var, "database:")) {
+	   config->output_thread_flag = 1;
 	   sagan_var = strtok_r(NULL, ",", &tok);
 	
 	   /* Type (only "log" is used right now */
@@ -365,8 +384,21 @@ if (!strcmp(sagan_var, "unified2:")) {
 
 	      remspaces(sagan_var);
 
-	      if (!strcmp(sagan_var, "mysql" )) config->dbtype=1; 
-	      if (!strcmp(sagan_var, "postgresql" )) config->dbtype=2; 
+
+	      if (!strcmp(sagan_var, "mysql" )) { 
+#ifndef HAVE_LIBMYSQLCLIENT_R
+   	         sagan_log(config, 1,"MySQL support not found. Re-compile with MySQL support or disable in the sagan.conf.");
+#endif
+	         config->dbtype=1; 
+	         }
+
+	      if (!strcmp(sagan_var, "postgresql" )) { 
+#ifndef HAVE_LIBPQ
+		 sagan_log(config, 1,"PostgreSQL support not found. Re-compile with PostgreSQL support or disable in the sagan.conf.");
+#endif
+		 config->dbtype=2; 
+		 }
+
 
 	      sagan_var = strtok_r(NULL, ",", &tok);
 	      remrt(sagan_var);					/* rm NL */
@@ -398,6 +430,10 @@ if (!strcmp(sagan_var, "unified2:")) {
 		   snprintf(config->dbhost, sizeof(config->dbhost), "%s", ptmp);
 		   }
 
+		if (!strcmp(ptmp, "port")) {
+		   config->dbport = atoi(ptmp);
+		   }
+
 	      ptmp = strtok_r(NULL, "=", &tok);
 
 
