@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -49,8 +49,8 @@
 
 struct _SaganCounters *counters;
 
-extern struct ref_struct *refstruct;
-extern struct rule_struct *rulestruct;
+struct _Ref_Struct *refstruct;
+struct _Rule_Struct *rulestruct;
 
 void load_reference( _SaganDebug *debug,  _SaganConfig *config, const char *ruleset )  { 
 
@@ -77,7 +77,7 @@ while(fgets(refbuf, 1024, reffile) != NULL) {
      continue;
      } else {
      /* Allocate memory for references,  not comments */
-     refstruct = (ref_struct *) realloc(refstruct, (counters->refcount+1) * sizeof(ref_struct));
+     refstruct = (_Ref_Struct *) realloc(refstruct, (counters->refcount+1) * sizeof(_Ref_Struct));
      }
 
      firststring = strtok_r(refbuf, ":", &saveptr);
@@ -113,9 +113,13 @@ char *reflookup( int rulemem, int type ) {
 
 int i=0;
 int b=0;
+
 char *tmptok=NULL;
 char *tmp=NULL;
-char *tmp2=NULL;
+
+char reftype[25]="";
+char url[255]=""; 
+
 char refinfo[512]="";
 char refinfo2[512]="";
 char reftmp[2048]="";
@@ -124,22 +128,35 @@ char *ret=NULL;
 for (i=0; i < rulestruct[rulemem].ref_count + 1 ; i++ ) {
 
         strlcpy(refinfo, rulestruct[rulemem].s_reference[i], sizeof(refinfo));
-        tmp = strtok_r(refinfo, ",", &tmptok);
-        tmp2 = strtok_r(NULL, ",", &tmptok);
+	
+	tmp = strtok_r(refinfo, ",", &tmptok);
+
+	if ( tmp != NULL ) { 
+	   strlcpy(reftype, tmp, sizeof(reftype)); 
+	   } else { 
+	   return("");
+	   }
+
+	tmp  = strtok_r(NULL, ",", &tmptok);
+	
+	if ( tmp != NULL ) { 
+	   strlcpy(url, tmp, sizeof(url)); 
+	   } else {
+	   return("");
+	   }
 
- if ( tmp != NULL ) { 
 
     for ( b=0; b < counters->refcount; b++) {
 
-        if (!strcmp(refstruct[b].s_refid,  tmp)) {
-	   if ( type == 0 ) snprintf(refinfo2, sizeof(refinfo2), "[Xref => %s%s]",  refstruct[b].s_refurl, tmp2);
-	   if ( type == 1 ) snprintf(refinfo2, sizeof(refinfo2), "Reference:%s%s\n", refstruct[b].s_refurl, tmp2);
+        if (!strcmp(refstruct[b].s_refid,  reftype)) {
+	   if ( type == 0 ) snprintf(refinfo2, sizeof(refinfo2), "[Xref => %s%s]",  refstruct[b].s_refurl, url);
+	   if ( type == 1 ) snprintf(refinfo2, sizeof(refinfo2), "Reference:%s%s\n", refstruct[b].s_refurl, url);
 	   strlcat(reftmp,  refinfo2,  sizeof(reftmp));
            }
         } 
    }
-}
 
 ret=reftmp;
+
 return(ret);
 }
