@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -90,13 +90,14 @@ void sig_handler( _SaganSigArgs *args ) {
 		  case SIGSEGV:
 		  case SIGABRT:
 
-		  printf("%d <-\n", args[0].daemonize);
-
                   sagan_log(args->config, 0, "\n\n[Received signal %d. Sagan version %s shutting down]-------\n", sig, VERSION);
 		  sagan_statistics(args->config);
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-		  if ( args->config->dbtype != 0 ) record_last_cid(args->debug, args->config);
+
+		  /* last_cid != cid (has there been any alerts? */
+
+		  if ( args->config->dbtype != 0 && counters->last_cid != counters-> cid ) record_last_cid(args->debug, args->config, counters);
 #endif
 
 #ifdef HAVE_LIBPRELUDE
