@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -39,7 +39,7 @@ void sagan_usage(void)
 {
 fprintf(stderr, "\n--[Sagan version %s | Help/usage screen]--------------------------------\n\n", VERSION);
 fprintf(stderr, "-h, --help\t\tHelp (this screen).\n");
-fprintf(stderr, "-d, --debug [type]\tTypes: syslog, load");
+fprintf(stderr, "-d, --debug [type]\tTypes: syslog, load, fwsam");
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
 fprintf(stderr, ", sql");
@@ -89,4 +89,8 @@ fprintf(stderr, "* liblognorm (log normalization) support is included\n");
 fprintf(stderr, "* PLOG (syslog sniffer) support is included\n");
 #endif
 
+#ifdef HAVE_LIBDNET
+fprintf(stderr, "* libdnet (for unified2) support is included\n");
+#endif
+
 }
