@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -46,6 +46,9 @@
 #include <limits.h>
 #include <stdint.h>
 #include <inttypes.h>
+#include <arpa/inet.h>
+#include <netinet/in.h>
+#include <sys/socket.h>
 
 #include "sagan.h"
 
@@ -58,16 +61,6 @@
 #include <lognorm.h>
 #endif
 
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-#include "output-plugins/sagan-snort.h"
-#endif
-
-#ifdef HAVE_LIBPRELUDE
-#include <libprelude/prelude.h>
-#include "output-plugins/sagan-prelude.h"
-#endif
-
-
 #ifdef HAVE_LIBDNET
 #include "output-plugins/sagan-unified2.h"
 #endif
@@ -75,8 +68,7 @@
 #define OVECCOUNT 30
 
 struct _SaganCounters *counters;
-
-struct rule_struct *rulestruct;
+struct _Rule_Struct *rulestruct;
 struct class_struct *classstruct;
 
 sbool daemonize=0;
@@ -127,49 +119,30 @@ int option_index = 0;
 
 /* Passing Sagan events to output plugins */
 
-struct Sagan_Event *SaganEvent = NULL;
-SaganEvent = malloc(MAX_THREADS * sizeof(struct Sagan_Event));
+struct _Sagan_Event *SaganEvent = NULL;
+SaganEvent = malloc(MAX_THREADS * sizeof(struct _Sagan_Event));
+
+/***************************************************************************/
+/* pthread for the output plugins                                          */
+/***************************************************************************/
+
+pthread_t output_id[MAX_THREADS];
+pthread_attr_t thread_output_attr;
+pthread_attr_init(&thread_output_attr);
+pthread_attr_setdetachstate(&thread_output_attr,  PTHREAD_CREATE_DETACHED);
 
 /****************************************************************************/
-/* MySQL / PostgreSQL (snort) local variables			    */
+/* pthread for the processor plugins                                        */
 /****************************************************************************/
 
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
+pthread_t processor_id[MAX_THREADS];
+pthread_attr_t thread_processor_attr;
+pthread_attr_init(&thread_processor_attr);
+pthread_attr_setdetachstate(&thread_processor_attr,  PTHREAD_CREATE_DETACHED);
 
-uint64_t cid = 0;
-
-pthread_t threaddb_id[MAX_THREADS];
-pthread_attr_t thread_db_attr;
-
-pthread_attr_init(&thread_db_attr);
-pthread_attr_setdetachstate(&thread_db_attr,  PTHREAD_CREATE_DETACHED);
-
-#endif
 
 /****************************************************************************/
-/* Prelude support                                                          */
-/****************************************************************************/
-
-#ifdef HAVE_LIBPRELUDE
-pthread_t threadprelude_id[MAX_THREADS];
-pthread_attr_t thread_prelude_attr;
-pthread_attr_init(&thread_prelude_attr);
-pthread_attr_setdetachstate(&thread_prelude_attr,  PTHREAD_CREATE_DETACHED);
-#endif
-
-/****************************************************************************/
-/* libesmtp (SMTP/e-mail) local variables				    */
-/****************************************************************************/
-
-#ifdef HAVE_LIBESMTP
-pthread_t threademail_id[MAX_THREADS];
-pthread_attr_t thread_email_attr;
-pthread_attr_init(&thread_email_attr);
-pthread_attr_setdetachstate(&thread_email_attr,  PTHREAD_CREATE_DETACHED);
-#endif
-
-/****************************************************************************/
-/* libpcap/PLOG (syslog sniffer) local variables                                 */
+/* libpcap/PLOG (syslog sniffer) local variables                            */
 /****************************************************************************/
 
 #ifdef HAVE_LIBPCAP
@@ -183,6 +156,12 @@ pthread_attr_setdetachstate(&thread_pcap_attr,  PTHREAD_CREATE_DETACHED);
 /* Various local variables						    */
 /****************************************************************************/
 
+#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
+char *sqlout=NULL;
+char *sql=NULL;
+char sqltmp[MAXSQL];
+#endif
+
 /* Block all signals,  we create a signal handling thread */
 
 sigset_t signal_set;
@@ -197,12 +176,9 @@ pthread_attr_t key_thread_attr;
 pthread_attr_init(&key_thread_attr);
 pthread_attr_setdetachstate(&key_thread_attr,  PTHREAD_CREATE_DETACHED);
 
-/* External thread support */
-
-pthread_t threadext_id[MAX_THREADS];
-pthread_attr_t thread_ext_attr;
-pthread_attr_init(&thread_ext_attr);
-pthread_attr_setdetachstate(&thread_ext_attr,  PTHREAD_CREATE_DETACHED);
+struct sockaddr_in sa;
+char src_dns_lookup[20];
+int  dns_flag=0;
 
 sbool fifoerr=0;
 
@@ -225,6 +201,11 @@ int  dst_port;
 int src_porttmp[MAX_MSGSLOT];
 int dst_porttmp[MAX_MSGSLOT];
 
+int  after_count_by_src=0;
+int  after_count_by_dst=0;
+sbool after_flag=0;
+sbool after_log_flag=0;
+
 int  thresh_count_by_src=0;
 int  thresh_count_by_dst=0;
 sbool thresh_flag=0;
@@ -234,7 +215,11 @@ char  timet[20];
 struct thresh_by_src *threshbysrc = NULL;
 struct thresh_by_dst *threshbydst = NULL;
 
+struct after_by_src *afterbysrc = NULL;
+struct after_by_dst *afterbydst = NULL;
+
 uint64_t thresh_oldtime_src;
+uint64_t after_oldtime_src;
 
 char fip[MAXIP];
 
@@ -304,7 +289,7 @@ int match=0;
 int pcrematch=0;
 
 time_t t;
-struct tm *now;
+struct tm *now, *run;
 
 /* Allocate and clear memory for global structs */
 
@@ -320,9 +305,17 @@ struct _SaganSigArgs *sigargs;
 sigargs = malloc(sizeof(_SaganSigArgs));
 memset(sigargs, 0, sizeof(_SaganSigArgs));
 
+struct _SaganDNSCache *dnscache; 
+dnscache = malloc(sizeof(_SaganDNSCache)); 
+memset(dnscache, 0, sizeof(_SaganDNSCache));
+
 counters = malloc(sizeof(_SaganCounters));
 memset(counters, 0, sizeof(_SaganCounters));
 
+t = time(NULL);
+run=localtime(&t);
+strftime(config->sagan_startutime, sizeof(config->sagan_startutime), "%s",  run);
+
 
 snprintf(config->sagan_config, sizeof(config->sagan_config), "%s", CONFIG_FILE_PATH);
 
@@ -347,6 +340,7 @@ while ((c = getopt_long(argc, argv, short_options, long_options, &option_index))
 
               if (strstr(optarg, "syslog")) debug->debugsyslog=1;
               if (strstr(optarg, "load")) debug->debugload=1;
+	      if (strstr(optarg, "fwsam")) debug->debugfwsam=1;
 
 #ifdef HAVE_LIBLOGNORM
 	      if (strstr(optarg, "normalize" )) debug->debugnormalize=1;
@@ -471,28 +465,40 @@ removelockfile(config);
 sagan_log(config, 1, "[%s, line %d] Can't open %s!", __FILE__, __LINE__, config->sagan_alert_filepath);
 }
 
-if ( config->sagan_ext_flag ) sagan_log(config, 0, "Max external threads : %d", config->max_external_threads);
+sagan_log(config, 0, "Max Output Threads   : %" PRIu64 "", config->max_output_threads);
+sagan_log(config, 0, "Max Processor Threads: %" PRIu64 "", config->max_processor_threads);
 
-#ifdef HAVE_LIBESMTP
-if ( config->sagan_esmtp_flag ) sagan_log(config, 0, "Max SMTP threads     : %d", config->max_email_threads);
-#endif
+sagan_log(config, 0, "");
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-
 if ( config->dbtype ) { 
 
-sagan_log(config, 0, "Max database threads : %d", config->maxdb_threads);
+config->endian = checkendian();
 
 db_connect(config);
 
 get_sensor_id( debug, config ); 
 sagan_log(config, 0, "Sensor ID            : %d", config->sensor_id);
-cid = get_cid( debug, config );
-cid++;
-counters->sigcid=cid;
-sagan_log(config, 0, "Next CID             : %" PRIu64 "", cid);
+counters->cid = get_cid( debug, config ) + 1; 
+
+snprintf(sqltmp, sizeof(sqltmp), "SELECT MAX(cid) FROM event WHERE sid=%d", config->sensor_id);
+sql=sqltmp;
+sqlout = db_query(debug,  config, sql);
+
+sagan_log(config, 0, "Next CID             : %" PRIu64 "", counters->cid);
+
+/* Check the event table and compare sensor.last_cid with event_cid.  If there's a 
+ * mismatch,  we correct it  - Champ Clark 03/30/2012 */ 
+
+if ( atol(sqlout) != counters->cid ) {
+   sagan_log(config, 2, "Inconsistent cid information for sid=%d.  Recovering by rolling forward to cid=%d", config->sensor_id, atol(sqlout) );
+   counters->cid = atol(sqlout);
+   record_last_cid(debug, config, counters);
+   }
+
+counters->last_cid = counters->cid;	/* Use to determine if a change has happened in sagan_siganl.c */
+} 
 
-}
 #endif
 
 #ifdef HAVE_LIBPRELUDE
@@ -500,7 +506,6 @@ sagan_log(config, 0, "Next CID             : %" PRIu64 "", cid);
 if ( config->sagan_prelude_flag ) {
 
 sagan_log(config, 0, "Prelude profile: %s", config->sagan_prelude_profile);
-sagan_log(config, 0, "Max Prelude threads: %d", config->max_prelude_threads);
 sagan_log(config, 0, "");  /* libprelude dumps some information.  This is to make it pretty */
 
 PreludeInit(config);
@@ -528,7 +533,7 @@ sagan_log(config, 0, "");
 sagan_log(config, 0, " ,-._,-. 	-*> Sagan! <*-");
 sagan_log(config, 0, " \\/)\"(\\/	Version %s", VERSION);
 sagan_log(config, 0, "  (_o_)	Champ Clark III & The Quadrant InfoSec Team [quadrantsec.com]");
-sagan_log(config, 0, "  /   \\/)	Copyright (C) 2009-2011 Quadrant Information Security, et al.");
+sagan_log(config, 0, "  /   \\/)	Copyright (C) 2009-2012 Quadrant Information Security, et al.");
 sagan_log(config, 0, " (|| ||) 	Using PCRE version: %s", pcre_version());
 sagan_log(config, 0, "  oo-oo     Sagan is processing events.....");
 sagan_log(config, 0, "");
@@ -620,24 +625,62 @@ while(1) {
                 {
 
 		counters->sagantotal++;
-
-		/* We have to check for values be "NULL" in the event that
-		 * the program generating the message did so incorrectly
-		 * *cough* Asterisk *cough*.  So we do a little checking 
-		 * here.  If we 'see' a bad valid,  we attempt to correct 
-		 * it */
-
-		/* If fifoerr is set,  we've likely lost our FIFO writer. 
-		 * If that's the case,  don't report because it's useless
-		 * information & will fill our logs */
-
 		syslog_host = strtok_r(syslogstring, "|", &tok);
-		
-		if (syslog_host == NULL ) { 
-		   syslog_host = "SAGAN: HOST ERROR"; 
-		   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'host'");
+
+		/* If we're using DNS (and we shouldn't be!),  we start DNS checks and lookups
+		 * here.  We cache both good and bad lookups to not over load our DNS server(s).
+		 * The only way DNS cache can be cleared is to restart Sagan */
+
+		if (config->syslog_src_lookup ) { 
+		   if ( inet_pton(AF_INET, syslog_host, &(sa.sin_addr)) == 0 ) { 	/* Is inbound a valid IP? */
+		      dns_flag=0;							
+
+		   for(i=0; i <= counters->dns_cache_count ; i++) {			/* Check cache first */
+		      if (!strcmp( dnscache[i].hostname, syslog_host)) { 
+		          syslog_host = dnscache[i].src_ip;
+		          dns_flag=1;
+		          } 
 		   }
+
+		   /* If entry was not found in cache,  look it up */
+
+		   if ( dns_flag == 0 ) { 
+
+		      /* Do a DNS lookup */
+		      snprintf(src_dns_lookup, sizeof(src_dns_lookup), "%s", dns_lookup(config, syslog_host));
+		    
+		      /* Invalid lookups get the config->sagan_host value */
+
+		      if (!strcmp(src_dns_lookup, "0" )) { 
+		         snprintf(src_dns_lookup, sizeof(src_dns_lookup), "%s", config->sagan_host);
+		  	 counters->dns_miss_count++; 
+			 }
+
+
+                    /* Add entry to DNS Cache */
+
+                    dnscache = (_SaganDNSCache *) realloc(dnscache, (counters->dns_cache_count+1) * sizeof(_SaganDNSCache));
+                    snprintf(dnscache[counters->dns_cache_count].hostname, sizeof(dnscache[counters->dns_cache_count].hostname), "%s", syslog_host);
+                    snprintf(dnscache[counters->dns_cache_count].src_ip, sizeof(dnscache[counters->dns_cache_count].src_ip), "%s",  src_dns_lookup);
+                    counters->dns_cache_count++; 
+                    syslog_host = src_dns_lookup; 
+
+		    }
+	       }
 		
+	} else { 
+
+	        /* We check to see if values from our FIFO are valid.  If we aren't doing DNS related 
+		 * stuff (above),  we start basic check with the syslog_host */
+
+                if (syslog_host == NULL || inet_pton(AF_INET, syslog_host, &(sa.sin_addr)) == 0  ) { 
+                   syslog_host = config->sagan_host;
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'host' (replaced with %s)", config->sagan_host);
+                   }
+	       }
+
+		/* We know check the rest of the values */
+
 		syslog_facility=strtok_r(NULL, "|", &tok);
 		if ( syslog_facility == NULL ) { 
 		   syslog_facility = "SAGAN: FACILITY ERROR";
@@ -685,13 +728,19 @@ while(1) {
                 if ( syslog_msg == NULL ) {
                    syslog_msg = "SAGAN: MESSAGE ERROR";
                    if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'message'\n");
+
+		   /* If the message is lost,  all is lost.  Typically,  you don't lose part of the message,  
+		    * it's more likely to lose all  - Champ Clark III 11/17/2011 */
+
+		   counters->sagan_log_drop++; 
+
                    }
 
 
                /* Strip any \n or \r from the syslog_msg */
 
-               syslog_msg[strcspn ( syslog_msg, "\n" )] = '\0';
-               syslog_msg[strcspn ( syslog_msg, "\r" )] = '\0';
+               if ( strcspn ( syslog_msg, "\n" ) < strlen(syslog_msg) ) 
+                  syslog_msg[strcspn ( syslog_msg, "\n" )] = '\0';
 
 
 if (debug->debugsyslog) sagan_log(config, 0, "%s|%s|%s|%s|%s|%s|%s|%s|%s", syslog_host, syslog_facility, syslog_priority, syslog_level, syslog_tag, syslog_date, syslog_time, syslog_program, syslog_msg);
@@ -940,6 +989,101 @@ if (uid != NULL ) {
 if (!strcmp(ip_src, "127.0.0.1" )) ip_src=config->sagan_host;
 if (!strcmp(ip_dst, "127.0.0.1" )) ip_dst=config->sagan_host;
 
+/*********************************************************/
+/* After - Similar to thresholding,  but the opposite    */
+/* direction - ie - alert _after_ X number of events     */
+/*********************************************************/
+
+if ( rulestruct[b].after_src_or_dst != 0 ) {
+     
+      after_log_flag=1;
+
+      t = time(NULL);
+      now=localtime(&t);
+      strftime(timet, sizeof(timet), "%s",  now);
+
+      /* After by source IP address */
+
+      if ( rulestruct[b].after_src_or_dst == 1 ) {
+         after_flag = 0;
+
+         for (i = 0; i < after_count_by_src; i++ ) {
+             if (!strcmp( afterbysrc[i].ipsrc, ip_src ) && !strcmp(afterbysrc[i].sid, rulestruct[b].s_sid )) {
+                after_flag=1;
+                afterbysrc[i].count++;
+                after_oldtime_src = atol(timet) - afterbysrc[i].utime;
+                afterbysrc[i].utime = atol(timet);
+                if ( after_oldtime_src > rulestruct[b].after_seconds ) {
+                   afterbysrc[i].count=1;
+                   afterbysrc[i].utime = atol(timet);
+                   after_log_flag=1;
+                   }
+
+                if ( rulestruct[b].after_count < afterbysrc[i].count )
+                        {
+                        after_log_flag = 0;
+                        sagan_log(config, 0, "After SID %s by source IP address. [%s]", afterbysrc[i].sid, ip_src);
+                        counters->after_total++;
+                        }
+
+            }
+          }
+	}
+
+
+         /* If not found,  add it to the array */
+
+         if ( after_flag == 0 ) {
+            afterbysrc = (after_by_src *) realloc(afterbysrc, (after_count_by_src+1) * sizeof(after_by_src));
+            snprintf(afterbysrc[after_count_by_src].ipsrc, sizeof(afterbysrc[after_count_by_src].ipsrc), "%s", ip_src);
+            snprintf(afterbysrc[after_count_by_src].sid, sizeof(afterbysrc[after_count_by_src].sid), "%s", rulestruct[b].s_sid );
+            afterbysrc[after_count_by_src].count = 1;
+            afterbysrc[after_count_by_src].utime = atol(timet);
+            after_count_by_src++;
+            }
+
+      /* After by destination IP address */
+
+        if ( rulestruct[b].after_src_or_dst == 2 ) {
+            
+	    after_flag = 0;
+
+        /* Check array for matching src / sid */
+
+        for (i = 0; i < after_count_by_dst; i++ ) {
+                if (!strcmp( afterbydst[i].ipdst, ip_dst ) && !strcmp(afterbydst[i].sid, rulestruct[b].s_sid )) {
+                   after_flag=1;
+                   afterbydst[i].count++;
+                   after_oldtime_src = atol(timet) - afterbydst[i].utime;
+                   afterbydst[i].utime = atol(timet);
+                      if ( after_oldtime_src > rulestruct[b].after_seconds ) {
+                         afterbydst[i].count=1;
+                         afterbydst[i].utime = atol(timet);
+                         after_log_flag=1;
+                         }
+
+        if ( rulestruct[b].after_count < afterbydst[i].count ) {
+           after_log_flag = 0;
+           sagan_log(config, 0, "After SID %s by destination IP address. [%s]", afterbysrc[i].sid, ip_dst);
+           counters->after_total++;
+           }
+         }
+       }
+
+        /* If not found,  add it to the array */
+
+        if ( after_flag == 0 ) {
+           afterbydst = (after_by_dst *) realloc(afterbydst, (after_count_by_dst+1) * sizeof(after_by_dst));
+           snprintf(afterbydst[after_count_by_dst].ipdst, sizeof(afterbydst[after_count_by_dst].ipdst), "%s", ip_dst);
+           snprintf(afterbydst[after_count_by_dst].sid, sizeof(afterbydst[after_count_by_dst].sid), "%s", rulestruct[b].s_sid );
+           afterbydst[after_count_by_dst].count = 1;
+           afterbydst[after_count_by_dst].utime = atol(timet);
+           after_count_by_dst++;
+           }
+        }
+
+} /* End of After */
+
 
 thresh_log_flag = 0;
 
@@ -947,7 +1091,7 @@ thresh_log_flag = 0;
 /* Thresh holding                                        */
 /*********************************************************/
 
-if ( rulestruct[b].threshold_type != 0 ) { 
+if ( rulestruct[b].threshold_type != 0 && after_log_flag == 0) { 
 
       t = time(NULL);
       now=localtime(&t);
@@ -1015,7 +1159,7 @@ if ( rulestruct[b].threshold_type != 0 ) {
 
 	if ( rulestruct[b].threshold_count < threshbydst[i].count ) {
 	   thresh_log_flag = 1;
-	   sagan_log(config, 0, "Threshold SID %s by source IP address. [%s]", threshbysrc[i].sid, ip_dst);
+	   sagan_log(config, 0, "Threshold SID %s by destination IP address. [%s]", threshbysrc[i].sid, ip_dst);
 	   counters->threshold_total++;
 	   }
          }
@@ -1034,13 +1178,16 @@ if ( rulestruct[b].threshold_type != 0 ) {
         }
 }  /* End of thresholding */
 
+
 /****************************************************************************/
 /* Populate the SaganEvent array with the information needed.  This info    */
 /* will be passed to the threads.  No need to populate it _if_ we're in a   */
 /* threshold state.                                                         */
 /****************************************************************************/
 
-if ( thresh_log_flag == 0 ) { 
+/* Check for thesholding & "after" */
+
+if ( thresh_log_flag == 0 && after_log_flag == 0 ) { 
 
 threadid++;
 if ( threadid >= MAX_THREADS ) threadid=0;
@@ -1089,149 +1236,74 @@ SaganEvent[threadid].config    = 	config;
 
 }
 
+/***************************************************************************/
+/* Output plugins that cannot be threaded and require little I/O (almost   */
+/* no I/O blocking) - IE - unified2/ASCII alerts                           */
+/***************************************************************************/
 
-/* Log alert to alert.log file */
+/* If thresholding isn't happening,  send to output plugins */
 
-if ( thresh_log_flag == 0 ) sagan_alert( &SaganEvent[threadid] );
+if ( thresh_log_flag == 0 && after_log_flag == 0 ) { 
+
+sagan_alert( &SaganEvent[threadid] );	/* 
 
 /* Log to unified2 output (if enabled and have libdnet). */
 
 #ifdef HAVE_LIBDNET
-
 if ( config->sagan_unified2_flag ) {
-
-if ( thresh_log_flag == 0 ) Sagan_Unified2( &SaganEvent[threadid] );
-if ( thresh_log_flag == 0 ) Sagan_Unified2LogPacketAlert( &SaganEvent[threadid] );
-
-}
-
-#endif
-
-/****************************************************************************/
-/* Prelude framework thread call (libprelude                                */
-/****************************************************************************/
-
-#if HAVE_LIBPRELUDE
-
-if ( config->sagan_prelude_flag == 1 && thresh_log_flag == 0 ) {
-	
-if ( counters->threadpreludec < config->max_prelude_threads ) {
-	
-	counters->threadpreludec++;
-
-	if ( counters->threadpreludec > counters->threadmaxpreludec ) counters->threadmaxpreludec=counters->threadpreludec;
-
-	if ( pthread_create ( &threadprelude_id[threadid], &thread_prelude_attr, (void *)sagan_prelude, &SaganEvent[threadid] ) ) { 
-		removelockfile(config);
-	        sagan_log(config, 1, "[%s, line %d] Error creating Prelude thread", __FILE__, __LINE__);
-	        } 
-	      	 } else { 
-                counters->sagandrop++;
-                counters->saganpreludedrop++;
-                sagan_log(config, 0, "Prelude thread call handler: Out of threads\n");
-              	}
+if ( thresh_log_flag == 0 && after_log_flag == 0 ) Sagan_Unified2( &SaganEvent[threadid] );
+if ( thresh_log_flag == 0 && after_log_flag == 0 ) Sagan_Unified2LogPacketAlert( &SaganEvent[threadid] );
 }
 #endif
 
-
 /****************************************************************************/
-/* libesmtp thread call (SMTP/email)                                        */
+/* Output Plugin Threads - This is for output plugins like SQL/etc that     */
+/* might cause some I/O blocking                                            */
 /****************************************************************************/
 
-#ifdef HAVE_LIBESMTP
+if ( config->output_thread_flag )  {
 
-/* Has e-mail been turned on? */
+if (  counters->thread_output_counter < config->max_output_threads ) {
 
-if ( config->sagan_esmtp_flag == 1 && thresh_log_flag == 0 ) {
+        counters->thread_output_counter++;
 
-   /* If so,  this rule based (email:) or configuration based (send-to) */
-   
-   if ( rulestruct[b].email_flag  || config->sagan_sendto_flag ) { 
-		  
-	/* E-mail only if over min_email_priority */ 
-
-	if ( config->min_email_priority >= rulestruct[b].s_pri || config->min_email_priority == 0 ) { 
-
-		if ( counters->threademailc < config->max_email_threads ) { 
-		  
-		    counters->threademailc++;
-
-		    if ( counters->threademailc > counters->threadmaxemailc ) counters->threadmaxemailc=counters->threademailc;
-
-                    if ( pthread_create( &threademail_id[threadid], &thread_email_attr, (void *)sagan_esmtp_thread, &SaganEvent[threadid] ) ) {
-		       removelockfile(config);
-                       sagan_log(config, 1, "[%s, line %d] Error creating SMTP thread", __FILE__, __LINE__);
-                       }
-
-			} else { 
-		       counters->sagandrop++;
-		       counters->saganesmtpdrop++;
-		       sagan_log(config, 0, "SMTP thread call handler: Out of threads\n");
-          }
-      }
-   }
-}
-#endif
-		
-/****************************************************************************/
-/* External program thread call                                             */
-/****************************************************************************/
-
-if ( config->sagan_ext_flag == 1 && thresh_log_flag == 0 ) { 
-		   
-   if ( counters->threadextc < config->max_external_threads ) { 
-
-	counters->threadextc++;
-		   
-	if ( counters->threadextc > counters->threadmaxextc ) counters->threadmaxextc=counters->threadextc;
-	
-		if ( pthread_create( &threadext_id[threadid], &thread_ext_attr, (void *)sagan_ext_thread, &SaganEvent[threadid] ) ) { 
-		     removelockfile(config);
-		     sagan_log(config, 1, "[%s, line %d] Error creating external call thread", __FILE__, __LINE__);
-		     }
-		      } else {
-		     counters->saganexternaldrop++;
-		     counters->sagandrop++; 
-		     sagan_log(config, 0, "External thread call handler: Out of threads\n");
-		   }
+if ( pthread_create ( &output_id[threadid], &thread_output_attr, (void *)sagan_output, &SaganEvent[threadid] ) ) {
+	removelockfile(config);
+        sagan_log(config, 1, "[%s, line %d] Error creating output-plugin thread", __FILE__, __LINE__);
+	} 
+} else { 
+	counters->sagan_output_drop++;
+	sagan_log(config, 0, "[%s, line %d] sagan_output(): Out of threads\n", __FILE__, __LINE__);
+  }
 }
 
 
 /****************************************************************************/
-/* Snort database thread call                                               */
+/* Log Processors that might take valuable CPU time (Geo IP, etc)           */
+/* Not sure if this section will remain here or not.  We might end up with  */
+/* "processors" and "preprocessors" - Champ Clark III 04/04/2012            */
 /****************************************************************************/
 
+if ( config->processor_thread_flag ) { 
 
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
+if ( counters->thread_processor_counter < config->max_processor_threads ) {
 
-config->endian = checkendian();    // Needed for Snort output
+        counters->thread_processor_counter++;
 
-if ( config->dbtype != 0 && thresh_log_flag == 0 ) { 
-
-        counters->threaddbc++;
-
-  		if ( counters->threaddbc < config->maxdb_threads ) { 
-
-			   if ( counters->threaddbc > counters->threadmaxdbc ) counters->threadmaxdbc=counters->threaddbc;
-                
-		   		cid++; 
-		   		counters->sigcid=cid;
-
-				SaganEvent[threadid].cid = cid;
-
-				if ( pthread_create( &threaddb_id[threadid], &thread_db_attr, (void *)sagan_db_thread, &SaganEvent[threadid]) ) { 
-		    		   removelockfile(config);
-		    		   sagan_log(config, 1, "[%s, line %d] Error creating database thread.", __FILE__, __LINE__);
-		    		   }
-		    		    } else { 
-		    		   counters->sagansnortdrop++;
-		    		   counters->sagandrop++;
-		    		   sagan_log(config, 0, "Snort database thread handler: Out of threads");
-	        		   }
+if ( pthread_create ( &processor_id[threadid], &thread_processor_attr, (void *)sagan_processor, &SaganEvent[threadid] ) ) {
+        removelockfile(config);
+        sagan_log(config, 1, "[%s, line %d] Error creating processor-plugin thread", __FILE__, __LINE__);
+        }
+} else {
+        counters->sagan_processor_drop++;
+        sagan_log(config, 0, "[%s, line %d] sagan_processor(): Out of threads\n", __FILE__, __LINE__);
 }
-#endif
-	 	    
-} /* End of match */
+}
+
+
+
+  } /* End of threshold */
+ } /* End of match */
 } /* End of pcre match */
 
 match=0;  /* Reset match! */
