@@ -1,7 +1,7 @@
 /* $Id$ */
 /*
-** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2012 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2012 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -29,6 +29,7 @@
 #include "config.h"             /* From autoconf */
 #endif
 
+#include <stdio.h>
 #include <stdint.h> 
 #include <pcre.h>
 #include <time.h>
@@ -67,18 +68,29 @@ char *sagan_getfilename(char *);
 
 char *referencelookup( int );
 
+typedef struct _SaganDNSCache _SaganDNSCache;
+struct _SaganDNSCache { 
+	
+	char hostname[64]; 
+	char src_ip[20];
+};
+
 typedef struct _SaganCounters _SaganCounters;
 struct _SaganCounters { 
 
     uint64_t threshold_total;
+    uint64_t after_total;
     uint64_t sagantotal;
     uint64_t saganfound;
-    uint64_t sagandrop;
+    uint64_t sagan_output_drop;
+    uint64_t sagan_processor_drop;
+    uint64_t sagan_log_drop;
+    uint64_t dns_cache_count;
+    uint64_t dns_miss_count;
+    uint64_t fwsam_count;
 
-    uint64_t threadmaxextc;
-    uint64_t saganexternaldrop;
-
-    int      threadextc;
+    int	     thread_output_counter; 
+    int	     thread_processor_counter;
 
     int	     classcount;
     int      rulecount;
@@ -86,23 +98,8 @@ struct _SaganCounters {
     int      ruletotal;
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-    uint64_t sigcid;            /* For passing CID with signal */
-    uint64_t threadmaxdbc;
-    int	     threaddbc;
-
-    uint64_t sagansnortdrop;
-#endif
-
-#ifdef HAVE_LIBESMTP
-    int      threademailc;
-    uint64_t saganesmtpdrop;
-    uint64_t threadmaxemailc;
-#endif
-
-#ifdef HAVE_LIBPRELUDE
-    int      threadpreludec;
-    uint64_t threadmaxpreludec;
-    uint64_t saganpreludedrop;
+    uint64_t last_cid;
+    uint64_t cid;            /* For passing CID with signal */
 #endif
 
 #ifdef HAVE_LIBLOGNORM
@@ -116,6 +113,7 @@ struct _SaganDebug {
 
     sbool debugsyslog;
     sbool debugload;
+    sbool debugfwsam;
 
 #ifdef HAVE_LIBLOGNORM
     sbool debugnormalize;
@@ -154,11 +152,19 @@ struct _SaganConfig {
     char 	 sagan_rule_path[MAXPATH];
     char         sagan_host[MAXHOST];
     char         sagan_extern[MAXPATH];
-    uint64_t	 max_external_threads;
+    char	 sagan_startutime[20]; 			/* Records utime at startup */
+    
+    uint64_t     max_output_threads;
+    sbool	 output_thread_flag;
+
+    uint64_t	 max_processor_threads;
+    sbool	 processor_thread_flag;
+
     int		 sagan_port;
     int		 sagan_exttype;
     sbool	 sagan_ext_flag;
     sbool        disable_dns_warnings;
+    sbool	 syslog_src_lookup;
     int		 daemonize;
     int          sagan_proto;
 
@@ -168,7 +174,6 @@ struct _SaganConfig {
 /* libesmtp/SMTP support */
     
 #ifdef HAVE_LIBESMTP
-    uint64_t	max_email_threads;
     int		min_email_priority;
     char	sagan_esmtp_to[255];
     sbool	sagan_sendto_flag;
@@ -180,7 +185,6 @@ struct _SaganConfig {
 /* Prelude framework support */
 
 #ifdef HAVE_LIBPRELUDE
-    uint64_t 	max_prelude_threads;
     char	sagan_prelude_profile[255];
     sbool	sagan_prelude_flag;
 #endif
@@ -212,15 +216,18 @@ struct _SaganConfig {
     int		 dbtype;
     int          sagan_detail;
     int		 sensor_id;
-    uint64_t	 maxdb_threads;
     char	 sagan_hostname[MAXHOST];
     char	 sagan_filter[50];
     char         dbuser[MAXUSER];
     char         dbpassword[MAXPASS];
     char         dbname[50]; 
     char         dbhost[50];
+    unsigned int dbport;
 #endif
 
+    sbool        sagan_fwsam_flag;
+    char 	 sagan_fwsam_info[1024];
+
 };
 
 
@@ -230,16 +237,16 @@ char *parse_ip_simple( char * );
 int   parse_port_simple(_SaganConfig *, char * );
 
 /* Reference structure */
-typedef struct ref_struct ref_struct;
-struct ref_struct {
+typedef struct _Ref_Struct _Ref_Struct;
+struct _Ref_Struct {
 unsigned s_size_ref;
 char s_refid[512];
 char s_refurl[2048];
 };
 
 /* Classification strucure */
-typedef struct class_struct class_struct;
-struct class_struct {
+typedef struct _Class_Struct _Class_Struct;
+struct _Class_Struct {
 unsigned s_size_class;
 char s_shortname[512];
 char s_desc[512];
@@ -247,8 +254,8 @@ int  s_priority;
 };
 
 /* Rule structure */
-typedef struct rule_struct rule_struct;
-struct rule_struct {
+typedef struct _Rule_Struct _Rule_Struct;
+struct _Rule_Struct {
 unsigned s_size_rule;
 char s_msg[512];
 
@@ -286,9 +293,16 @@ int threshold_src_or_dst;	// 1 ==  src,  2 == dst
 int threshold_count;		
 int threshold_seconds;
 
+int after_src_or_dst;       // 1 ==  src,  2 == dst
+int after_count;
+int after_seconds;
+
+int fwsam_src_or_dst;		// 1 == src,  2 == dst
+unsigned long  fwsam_seconds;		
+
 };
 
-typedef struct Sagan_Event 
+typedef struct _Sagan_Event 
 {
 
         _SaganDebug *debug;
@@ -323,11 +337,8 @@ typedef struct Sagan_Event
         char *program;
         char *message;          /* msg + sysmsg? */
 
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-	uint64_t cid;
-#endif
+} _SaganEvent;
 
-} SaganEvent;
 
 /* Thresholding structure by source */
 typedef struct thresh_by_src thresh_by_src;
@@ -349,6 +360,26 @@ uint64_t utime;
 char sid[512];
 };
 
+/* After structure by source */
+typedef struct after_by_src after_by_src;
+struct after_by_src {
+unsigned s_size_afterh_by_src;
+char ipsrc[64];
+int  count;
+uint64_t utime;
+char sid[512];
+};
+
+/* After structure by destination */
+typedef struct after_by_dst after_by_dst;
+struct after_by_dst {
+unsigned s_size_after_by_dst;
+char ipdst[64];
+int  count;
+uint64_t utime;
+char sid[512];
+};
+
 /****************************************************************************/
 /* MySQL & PostgreSQL support.  Support for Snort databases                 */
 /****************************************************************************/
@@ -360,12 +391,22 @@ char sid[512];
 #define MYSQL_PORT      3306
 
 char *sql_escape(_SaganConfig *, const char *, int );
-void sagan_db_thread( SaganEvent * );
+void sagan_db_thread( _SaganEvent * );
 int  ip2bit( _SaganConfig *, char * );
 char *fasthex(char *, int);
+int db_connect( _SaganConfig * );
+int  get_sensor_id ( _SaganDebug *, _SaganConfig *);
+uint64_t get_cid ( _SaganDebug *,  _SaganConfig * );
+char *db_query ( _SaganDebug *, _SaganConfig *,  char * );
 
 #endif
 
+#ifdef HAVE_LIBPRELUDE
+void PreludeInit( _SaganConfig *);
+void sagan_prelude( _SaganEvent * );
+#endif
+
+
 
 #ifdef HAVE_LIBLOGNORM
 /* liblognorm struct */
@@ -395,7 +436,7 @@ char filepath[MAXPATH];
 #define MAX_EMAILSIZE   15360		/* Largest e-mail that can be sent */
 
 const char *esmtp_cb (void **, int *, void *);
-void sagan_esmtp_thread( SaganEvent * );
+void sagan_esmtp_thread( _SaganEvent * );
 
 #endif
 
@@ -421,8 +462,8 @@ typedef struct _SaganSigArgs _SaganSigArgs;
 void plog_handler( _SaganSigArgs * );
 #endif
 
-void sagan_alert( SaganEvent * );
-void sagan_ext_thread( SaganEvent * );
+void sagan_alert( _SaganEvent * );
+void sagan_ext_thread( _SaganEvent * );
 
 void load_config( _SaganDebug *, _SaganConfig * );
 void sig_handler( _SaganSigArgs * );
@@ -436,3 +477,8 @@ void sagan_statistics( _SaganConfig * );
 void key_handler( _SaganConfig * );
 void sagan_droppriv( _SaganConfig *, const char *);
 char *dns_lookup( _SaganConfig *, char *);
+
+void sagan_output( _SaganEvent * );
+void sagan_processor( _SaganEvent * );
+
+void sagan_fwsam( _SaganEvent * );
